#include <alio.h>
#ifdef WIN
#include <stdlib.h>
#endif /* WIN */
#include <string.h>
#include <ctype.h>

#define AL_ID(a,b,c,d) (((a)<<24) | ((b)<<16) | ((c)<<8) | (d))
#define SYSTEM_NONE	AL_ID('N','O','N','E')

#ifdef USE_ZLIB
PACKFILE *pack_fopen_zip_dir(const char* filename);
PACKFILE *pack_fopen_zip(const PACKFILE *zip, const char *filename);
int pack_fclose(PACKFILE *stream);
#endif /* USE_ZLIB */

PACKFILE *pack_fopen(const char *filename, const char *mode);
int pack_vfprintf(PACKFILE *stream, const char *format, va_list ap);
long pack_fwrite(const void *buff, long size, PACKFILE *stream);
int stdio_vsprintf(const char *str, const char *format, va_list ap);
int install_allegro(int system_id, int *errno_ptr, int (*atexit_fn)());
void allegro_exit();

static int io_errno;
static int io_atexit(){
	allegro_exit();
	return 0;
}

void init_alio(){
	(void)install_allegro(SYSTEM_NONE, &io_errno, io_atexit);
}

PACKFILE* multi_pack_fopen(const char *filename, const char *mode) {
     PACKFILE *file=NULL;
#ifdef USE_ZLIB
     char *zipname=NULL;
     int i=0;
     for(;filename[i];++i) {
        if(filename[i]=='.') {
             if(toupper(filename[i+1])=='Z'&&toupper(filename[i+2])=='I'&&toupper(filename[i+3])=='P') {
				 if(filename[i+4]!='\\'&&filename[i+4]!='/'&&filename[i+4]!='#')
					continue;
				zipname=malloc(i+1+4);
				memcpy(zipname,filename,i+4);
				zipname[i+4]=0;
				if(!zipname)
					return file;
				filename=filename+i+5;
				break;
			}
        }
     }
     for(i=0;filename[i]&&(filename[i]=='\\'||filename[i]=='/'||filename[i]=='#');++i);
     filename=filename+i;
     if(zipname&&*zipname) {
			PACKFILE *zip = pack_fopen_zip_dir(zipname);
			if(zip) {
				file = pack_fopen_zip(zip, filename);
				pack_fclose(zip);
			}
     }
     if(zipname)
	free(zipname);
    if(file==NULL)
#endif /* USE_ZLIB */
     file=pack_fopen(filename, mode);
     return file;
}

extern int __vfscanf(PACKFILE *stream, const char *format, va_list ap);

int pack_vfscanf(PACKFILE *stream, const char *format, va_list ap) {
	return __vfscanf(stream, format, ap);
}

int pack_fscanf(PACKFILE *stream, const char *format, ...) {
	int status;
    va_list ap;
	va_start(ap,format);
	status=pack_vfscanf(stream, format, ap);
	va_end(ap);	
	return status;
}

#define VPRF_BUF_MAX 8192
int pack_vfprintf(PACKFILE *stream, const char *format, va_list ap) {
	int status;
	char buf[VPRF_BUF_MAX];
	status=vsprintf(buf, format, ap);
	if(status>=0) {
		buf[status]=0;
		if(fwrite(buf,sizeof(char),status,stream)!=(unsigned int)status) 
			return -1;
	}
	return status;
}

int pack_fprintf(PACKFILE *stream, const char *format, ...) {
	int status;
    va_list ap;
	va_start(ap,format);
	status=pack_vfprintf(stream, format, ap);
	va_end(ap);	
	return status;
}
